//
//  ContentView.swift
//  naiyoup
//
//  Created by C Auto on 2021/5/27.
//

import SwiftUI


struct LoginInEdit: View {
    @Binding var isReg : Bool
    @EnvironmentObject var loginVMode : LoginViewModel
    @State private var showPassword = false
    var body: some View {
        
        if isReg {
            HStack{
                Image(systemName: "person")
                    .foregroundColor(.secondary)
                TextField("用户昵称",
                          text: $loginVMode.usernick)
            }
            .padding()
            .overlay(
                RoundedRectangle(cornerRadius: 20)
                    .stroke(Color.gray, lineWidth: 1)
            )
        }
        
        HStack{
            Image(systemName: "person")
                .foregroundColor(.secondary)
            TextField("email",
                      text: $loginVMode.username)
        }
        .padding()
        .overlay(
            RoundedRectangle(cornerRadius: 20)
                .stroke(Color.gray, lineWidth: 1)
        )
        
        HStack{
            Image(systemName: "lock")
                .foregroundColor(.secondary)
            //            TextField("输入你的密码", text: $loginVMode.password)
            if showPassword {
                TextField("Password",
                          text: $loginVMode.password)
            } else {
                SecureField("Password",
                            text: $loginVMode.password)
            }
            Button(action: {
                self.showPassword.toggle()
            }) {
                
                Image(systemName: "eye")
                    .foregroundColor(.secondary)
            }
        }
        .padding()
        .overlay(
            RoundedRectangle(cornerRadius: 20)
                .stroke(Color.gray, lineWidth: 1)
        )
        
    }
    
}




struct LoginInButton: View {
    
    @Binding var isReg : Bool
    
    @EnvironmentObject var loginVMode : LoginViewModel
    
    var body: some View{
        
        ZStack {
            if self.loginVMode.loading {
                ActivityIndicator(isAnimating: self.loginVMode.loading)
                    .configure { $0.color = .yellow}
            } else {
                if self.isReg {
                    Text("注册")
                } else {
                    Text("登陆")
                }
                
            }
            
        }
        .frame(minWidth: 0, maxWidth: .infinity)
        
        .padding()
        .foregroundColor(.white)
        .background(Color("ButtonGreean"))
        .cornerRadius(20)
        
        
        
        
        
    }
    
}

struct LoginView: View {
    
    @State var username : String = ""
    @State var password : String = ""
    @State var isReg : Bool = false
    @ObservedObject var LoginVmodel = LoginViewModel.user
    @State var isAlte : Bool  = false
    
    init() {
        UIApplication.statusBarBackgroundColor = UIColor.init(named: "BackgroundColor")
        UINavigationBar.appearance().backgroundColor = UIColor.init(named: "BackgroundColor")
        UINavigationBar.appearance().tintColor = UIColor.white
        UINavigationBar.appearance().barTintColor = UIColor.init(named: "BackgroundColor")
        UINavigationBar.appearance().titleTextAttributes = [.foregroundColor: UIColor.white]
        UINavigationBar.appearance().largeTitleTextAttributes = [.foregroundColor: UIColor.white]
        UINavigationBar.appearance().isTranslucent = false
    }
    
    var body: some View {
        
        
        
        ZStack{
            
            NavigationView{
                
                VStack{
                    
                    //                NavigationLink("",destination: MainView(),) {
                    //
                    //                }
                    LoginInEdit(isReg: $isReg).environmentObject(self.LoginVmodel)
                    Button(action: {
                        if self.LoginVmodel.islogin {
                            UIApplication.shared.windows.first?.rootViewController  = UIHostingController(rootView: MainView())
                        } else {
                            
                            if self.isReg  {
                                LoginVmodel.Reg()
                            } else {
                                LoginVmodel.login()
                            }
                           
                        }
                        
                        
                    }) {
                        
                    LoginInButton(isReg: $isReg).environmentObject(self.LoginVmodel)
                        
                    }.alert(isPresented: self.$LoginVmodel.isAlt){
                        Alert(title: Text("提示"), message: Text(self.LoginVmodel.errmsg), dismissButton: .default(Text("确定")))
                    }
                    .padding(.top)
                    
                   // NavigationLink("", destination: RegView(), isActive: $isReg)
                    HStack{
                        Text("还没有注册账号?")
                        
                        Button(action: {
                            print("注册账号")
                           self.isReg.toggle()
                            
                        }) {
                            
                            Text(self.isReg ? "登陆" : "注册账号")
                            
                        }
                    }
                    .padding()
                    
                    Spacer();
                }
                .padding()
                
                .navigationBarTitle("登陆", displayMode: .inline)
                .onAppear {
                    let controller = UIApplication.shared.windows[0].rootViewController as? MyHontingController
                    controller?.statusBarStyle = .lightContent
                    UITableView.appearance().tableFooterView = UIView()
                    UITableView.appearance().separatorStyle = .none

                }
                
                
            }
        }
    }
    
    
    
    
    
}

struct LoginView_Previews: PreviewProvider {
    static var previews: some View {
        LoginView()
    }
}
